# SPDX-FileCopyrightText: 2024 Alec Delaney
# SPDX-License-Identifier: MIT

"""Configuration settings for sphinx."""

import sphinx_rtd_theme

project = "cronberry"
copyright = "2024 Alec Delaney"
author = "Alec Delaney"

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.todo",
    # "autoapi.extension",
]

templates_path = ["_templates"]
exclude_patterns = []

# -- Options for Sphinx AutoAPI------------------------------------------------
# autoapi_dirs = ["../cronberry"]


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = "sphinx_rtd_theme"
html_theme_path = [sphinx_rtd_theme.get_html_theme_path(), "_static"]
# html_static_path = ["_static"]
